<?php

if ( ! defined( 'ABSPATH' ) ) exit;

class Easy_Social_Share_Buttons {

	/**
	 * The single instance of Easy_Social_Share_Buttons.
	 * @var 	object
	 * @access  private
	 * @since 	1.0.0
	 */
	private static $_instance = null;

	/**
	 * Settings class object
	 * @var     object
	 * @access  public
	 * @since   1.0.0
	 */
	public $settings = null;

	/**
	 * The version number.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $_version;

	/**
	 * The token.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $_token;

	/**
	 * The main plugin file.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $file;

	/**
	 * The main plugin directory.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $dir;

	/**
	 * The plugin assets directory.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $assets_dir;

	/**
	 * The plugin assets URL.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $assets_url;

	/**
	 * Suffix for Javascripts.
	 * @var     string
	 * @access  public
	 * @since   1.0.0
	 */
	public $script_suffix;

	/**
	 * Constructor function.
	 * @access  public
	 * @since   1.0.0
	 * @return  void
	 */
	public function __construct ( $file = '', $version = '1.0.0' ) {
		$this->_version = $version;
		$this->_token = 'easy_social_share_buttons';

		// Load plugin environment variables
		$this->file = $file;
		$this->dir = dirname( $this->file );
		$this->assets_dir = trailingslashit( $this->dir ) . 'assets';
		$this->assets_url = esc_url( trailingslashit( plugins_url( '/assets/', $this->file ) ) );

		$this->script_suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		register_activation_hook( $this->file, array( $this, 'install' ) );

		// Load frontend CSS
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ), 10 );

		// Load API for generic admin functions
		if ( is_admin() ) {
			$this->admin = new Easy_Social_Share_Buttons_Admin_API();
		}

		// Handle localisation
		$this->load_plugin_textdomain();
		add_action( 'init', array( $this, 'load_localisation' ), 0 );

		//Add post content filters
		add_filter('the_content', array( $this, 'add_share_buttons_to_post'));
		add_filter('the_content', array( $this, 'add_share_buttons_to_media'));

		//Create shortcode
		add_shortcode( 'ess_post', array($this, 'share_post_shortcode') );

	} // End __construct ()

	/**
	 * Load frontend CSS.
	 * @access  public
	 * @since   1.0.0
	 * @return void
	 */
	public function enqueue_styles () {
		if (get_option('ess_load_css')) {
			wp_register_style( $this->_token . '-frontend', esc_url( $this->assets_url ) . 'css/frontend.css', array(), $this->_version );
			wp_enqueue_style( $this->_token . '-frontend' );
		}
	} // End enqueue_styles ()

	/**
	 * Load plugin localisation
	 * @access  public
	 * @since   1.0.0
	 * @return  void
	 */
	public function load_localisation () {
		load_plugin_textdomain( 'easy-social-share-buttons', false, dirname( plugin_basename( $this->file ) ) . '/lang/' );
	} // End load_localisation ()

	/**
	 * Load plugin textdomain
	 * @access  public
	 * @since   1.0.0
	 * @return  void
	 */
	public function load_plugin_textdomain () {
	    $domain = 'easy-social-share-buttons';

	    $locale = apply_filters( 'plugin_locale', get_locale(), $domain );

	    load_textdomain( $domain, WP_LANG_DIR . '/' . $domain . '/' . $domain . '-' . $locale . '.mo' );
	    load_plugin_textdomain( $domain, false, dirname( plugin_basename( $this->file ) ) . '/lang/' );
	} // End load_plugin_textdomain ()

	/**
	 * Main Easy_Social_Share_Buttons Instance
	 *
	 * Ensures only one instance of Easy_Social_Share_Buttons is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @static
	 * @see Easy_Social_Share_Buttons()
	 * @return Main Easy_Social_Share_Buttons instance
	 */
	public static function instance ( $file = '', $version = '1.0.0' ) {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self( $file, $version );
		}
		return self::$_instance;
	} // End instance ()

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->_version );
	} // End __clone ()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup () {
		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?' ), $this->_version );
	} // End __wakeup ()

	/**
	 * Installation. Runs on activation.
	 * @access  public
	 * @since   1.0.0
	 * @return  void
	 */
	public function install () {
		$this->_log_version_number();
	} // End install ()

	/**
	 * Log the plugin version number.
	 * @access  public
	 * @since   1.0.0
	 * @return  void
	 */
	private function _log_version_number () {
		update_option( $this->_token . '_version', $this->_version );
	} // End _log_version_number ()

	/**
	 * Return the url of the current page.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_current_url() {
		global $wp;
		return home_url(add_query_arg(array(),$wp->request)) . '/';
	} // End get_current_url()

	/**
	 * Return the number of shares on Facebook.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_facebook_count($url) {
		//https://api.facebook.com/method/links.getStats?urls=%%URL%%&format=json

		$endpoint = 'https://api.facebook.com/method/links.getStats?urls=' . $url . '&format=json';

		// setup curl to make a call to the endpoint
		$session = curl_init($endpoint);

		// indicates that we want the response back rather than just returning a "TRUE" string
		curl_setopt($session, CURLOPT_RETURNTRANSFER, true);

		// execute GET and get the session back
		$result = json_decode(curl_exec($session));

		// close connection
		curl_close($session);

		return $result[0]->total_count;
	} // End get_facebook_count()

	/**
	 * Return the number of shares on Twitter.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_twitter_count($url) {
		//http://urls.api.twitter.com/1/urls/count.json?url=%%URL%%&callback=twttr.receiveCount  

		$endpoint = 'http://urls.api.twitter.com/1/urls/count.json?url=' . $url;

		$curl = curl_init();

		curl_setopt($curl, CURLOPT_URL, $endpoint);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-type: application/json'));

		# Get the response
		$response = curl_exec($curl);

		# Close connection
		curl_close($curl);

		# Return JSON
		$result = json_decode($response, true);

		return $result["count"];
	} // End get_twitter_count()

	/**
	 * Return the number of shares on Pinterest.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_pinterest_count($url) {
		//https://api.pinterest.com/v1/urls/count.json?callback=jsonp&url=%%URL%%

		$endpoint = 'https://api.pinterest.com/v1/urls/count.json?callback=jsonp&url=' . $url;

		$curl = curl_init();

		curl_setopt($curl, CURLOPT_URL, $endpoint);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-type: application/json'));

		# Get the response
		$response = curl_exec($curl);

		# Close connection
		curl_close($curl);

		# Return JSON
		$response = str_replace(array('jsonp(', ')'), '', $response);
		$result = json_decode($response, true);

		return $result['count'];
	} // End get_pinterest_count()

	/**
	 * Return the number of shares on Google Plus.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_google_count($url) {

		$html =  file_get_contents( "https://plusone.google.com/_/+1/fastbutton?url=" . $url );
		$doc = new DOMDocument();   
		libxml_use_internal_errors(true);
		$doc->loadHTML($html);
		$counter=$doc->getElementById('aggregateCount');
		libxml_clear_errors();
		return $counter->nodeValue;
	} // End get_google_count()

	/**
	 * Create and return a post excerpt from a post ID outside of the loop.
	 * A similar function in Wordpress was deprecated.
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_excerpt_by_id($post_id){
	    $the_post = get_post($post_id); //Gets post ID
	    $the_excerpt = $the_post->post_content; //Gets post_content to be used as a basis for the excerpt
	    $excerpt_length = 35; //Sets excerpt length by word count
	    $the_excerpt = strip_tags(strip_shortcodes($the_excerpt)); //Strips tags and images
	    $words = explode(' ', $the_excerpt, $excerpt_length + 1);

	    if(count($words) > $excerpt_length) :
	        array_pop($words);
	        array_push($words, '…');
	        $the_excerpt = implode(' ', $words);
	    endif;

	    return $the_excerpt;
	} //End get_excerpt_by_id()

	/**
	 * Encode the url query parameters in hex to preserve special characters
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function mb_rawurlencode($url){
		$encoded='';
		$length=mb_strlen($url);

		for ($i=0;$i<$length;$i++) {
			$encoded.='%'.wordwrap(bin2hex(mb_substr($url,$i,1)),2,'%',true);
		}

		return $encoded;
	} // End mb_rawurlencode()

	/**
	 * Generate buttons
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function get_button_html($social_site, $post_id, $show_count, $image_url = null, $image_id = null) {

		$title = $this->mb_rawurlencode( get_the_title( $post_id ) );
		$description = $this->mb_rawurlencode( $this->get_excerpt_by_id( $post_id ) );
		$link = rawurlencode( get_permalink( $post_id ) );

		if ( get_option('permalink_structure') ) {
			$slash = '';	
		} else {
			// if pretty permalinks are not enabled add slash
			$slash = '/';
		}

		$direct_link = get_permalink( $post_id ) . ($image_id ? $slash . '#' . $image_id : '');
		$current_url = $this->get_current_url();
		$button = '';

		switch ($social_site) {
			case 'facebook':

			    $facebook_share_url = 'https://www.facebook.com/dialog/feed?';
			    $facebook_share_url .= 'app_id=' . esc_html( get_option('ess_facebook_app_id') );
			    $facebook_share_url .= '&amp;display=popup';
			    $facebook_share_url .= '&amp;caption=' . $title;
			    $facebook_share_url .= '&amp;link=' . $link;
			    $facebook_share_url .= '&amp;description=' . $description;
			    $facebook_share_url .= '&amp;redirect_uri=' . $link;

			    if (!empty($image_url)) {
			    	$facebook_share_url .= '&amp;picture=' . rawurlencode( $image_url );
			    }

				$button = '<a class="ess-button ess-button--facebook" href="' . $facebook_share_url . '" onclick="window.open(this.href, \'facebookwindow\',\'left=20,top=20,width=600,height=700,toolbar=0,resizable=1\'); return false;" title="Share on Facebook" target="_blank">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<i class="fa fa-facebook"></i>' . "\n";
				$button .= '<span class="ess-share-text">Share</span>' . "\n";
				$button .= '</div>' . "\n";
				
				if ($show_count) {
					$button .= '<span class="ess-social-count">' . $this->get_facebook_count( $current_url ) . '</span>';
				}

				$button .= '</a>' . "\n";
			break;

			case 'twitter':
				$twitter_url = 'http://twitter.com/intent/tweet?';
				$twitter_url .= 'text=' . $title . '%20' . $link;

				$button = '<a class="ess-button ess-button--twitter" href="' . $twitter_url .'" onclick="window.open(this.href, \'twitterwindow\',\'left=20,top=20,width=600,height=300,toolbar=0,resizable=1\'); return false;" title="Tweet" target="_blank">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<i class="fa fa-twitter"></i>' . "\n";
				$button .= '<span class="ess-share-text">Tweet</span>' . "\n";
				$button .= '</div>' . "\n";
				
				if ($show_count) {
					$button .= '<span class="ess-social-count">' . $this->get_twitter_count( $current_url ) . '</span>';
				}

				$button .= '</a>' . "\n";
			break;
			
			case 'gplus':
				$google_url = 'https://plus.google.com/share?';
				$google_url .= 'url=' . $link;

				$button = '<a class="ess-button ess-button--gplus" href="' . $google_url .'" onclick="javascript:window.open(this.href,\'googlepluswindow\', \'menubar=no,toolbar=no,resizable=yes,scrollbars=yes,height=600,width=600\');return false;" title="Share on Google" target="_blank">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<i class="fa fa-google-plus"></i>' . "\n";
				$button .= '<span class="ess-share-text">Share</span>' . "\n";
				$button .= '</div>' . "\n";
				
				if ($show_count) {
					$button .= '<span class="ess-social-count">' . $this->get_google_count( $current_url ) . '</span>';
				}

				$button .= '</a>' . "\n";
			break;

			case 'pinterest':
				$pinterest_url = 'http://pinterest.com/pin/create/bookmarklet/';
				$pinterest_url .= '?media=' . rawurlencode( $image_url );
				$pinterest_url .= '&amp;url=' . $link;
				$pinterest_url .= '&amp;is_video=false';
				$pinterest_url .= '&amp;description=' . $title . rawurlencode(' – ') . $description;

				$button = '<a class="ess-button ess-button--pinterest" href="' . $pinterest_url . '" onclick="window.open(this.href, \'pinterestwindow\',\'left=20,top=20,width=750,height=750,toolbar=0,resizable=1\');" title="Pin" target="_blank">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<i class="fa fa-pinterest"></i>' . "\n";
				$button .= '<span class="ess-share-text">Pin</span>' . "\n";
				$button .= '</div>' . "\n";
				
				if ($show_count) {
					$button .= '<span class="ess-social-count">' . $this->get_pinterest_count( $current_url ) . '</span>';
				}

				$button .= '</a>' . "\n";
			break;

			case 'email':

				$email_url = 'mailto:';
				$email_url .= '?subject=' . $title;
				$email_url .= '&amp;body=' . $description . '%20%20' . rawurlencode($direct_link);

				$button = '<a class="ess-button ess-button--email" href="' . $email_url . '" title="Email">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<i class="fa fa-envelope"></i>' . "\n";
				$button .= '<span class="ess-share-text">Email</span>' . "\n";
				$button .= '</div>' . "\n";
				$button .= '</a>' . "\n";
			break;

			case 'link':

				$button = '<a class="ess-button ess-button--link" href="' . $direct_link . '" title="Share Direct Link" onclick="event.preventDefault(); ">' . "\n";
				$button .= '<div class="ess-button-inner">' . "\n";
				$button .= '<svg class="ess-icon" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 1024 1024"><g><path class="path1" d="M440.236 635.766c-13.31 0-26.616-5.076-36.77-15.23-95.134-95.136-95.134-249.934 0-345.070l192-192c46.088-46.086 107.36-71.466 172.534-71.466s126.448 25.38 172.536 71.464c95.132 95.136 95.132 249.934 0 345.070l-87.766 87.766c-20.308 20.308-53.23 20.308-73.54 0-20.306-20.306-20.306-53.232 0-73.54l87.766-87.766c54.584-54.586 54.584-143.404 0-197.99-26.442-26.442-61.6-41.004-98.996-41.004s-72.552 14.562-98.996 41.006l-192 191.998c-54.586 54.586-54.586 143.406 0 197.992 20.308 20.306 20.306 53.232 0 73.54-10.15 10.152-23.462 15.23-36.768 15.23z"></path><path class="path2" d="M256 1012c-65.176 0-126.45-25.38-172.534-71.464-95.134-95.136-95.134-249.934 0-345.070l87.764-87.764c20.308-20.306 53.234-20.306 73.54 0 20.308 20.306 20.308 53.232 0 73.54l-87.764 87.764c-54.586 54.586-54.586 143.406 0 197.992 26.44 26.44 61.598 41.002 98.994 41.002s72.552-14.562 98.998-41.006l192-191.998c54.584-54.586 54.584-143.406 0-197.992-20.308-20.308-20.306-53.232 0-73.54 20.306-20.306 53.232-20.306 73.54 0.002 95.132 95.134 95.132 249.932 0.002 345.068l-192.002 192c-46.090 46.088-107.364 71.466-172.538 71.466z"></path></g></svg>' . "\n";
				$button .= '<span class="ess-share-text">' . $direct_link . '</span>' . "\n";
				$button .= '</div>' . "\n";
				$button .= '<div class="ess-share-link-wrap">' . "\n";
				$button .= '<input class="ess-share-link" type="text" value="' . $direct_link . '" onclick="this.setSelectionRange(0, this.value.length)"/>' . "\n";
				$button .= '</div>' . "\n";
				$button .= '</a>' . "\n";
			break;

			default:
			break;
		}

		return $button;
	}// End get_button_html()

	/**
	 * Build the html the share button component
	 * @access  private
	 * @since   1.0.0
	 * @return  string
	 */
	private function build_share_buttons($button_type) {
		global $post;

		$html = '';

		$sites = get_option('ess_social_sites');

		if ( is_array( $sites ) ) {

			$extra_classes = '';
			$show_count = false;

			switch ($button_type) {
				case 'text':
					$extra_classes = 'ess-buttons--text';
					break;
				case 'count':
					$extra_classes = 'ess-buttons--count';
					$show_count = true;
					break;
			}

			$html = '<ul class="ess-buttons ' . $extra_classes . '">' . "\n";

			// Get post thumbnail if it has one
			$thumbnail_url = '';
			$thumbnail_id = get_post_thumbnail_id( $post->ID );

			if ( !empty( $thumbnail_id ) ) {
				$thumbnail = wp_get_attachment_image_src($thumbnail_id,'large', true);

				if ( $thumbnail ) {
					$thumbnail_url = $thumbnail[0];
				}
			}
		
			foreach($sites as $site) {

				$button = $this->get_button_html($site, $post->ID, $show_count, $thumbnail_url);

				$html .= '<li>' . $button . '</li>' . "\n";
			}

			$html .= '</ul>' . "\n";
		}

		return $html;
	} // End build_share_buttons()

	/**
	 * Add the social sharing buttons either before or after the post content.
	 * @access  public
	 * @since   1.0.0
	 * @return  string
	 */
	public function add_share_buttons_to_post($content) {
		if (is_single()) {
			$location = get_option('ess_share_location');

			$button_type = get_option( 'ess_share_type' );

			if ( is_array( $location ) ) {
				foreach($location as $position) {
					if ($position == 'before') {
						$content = $this->build_share_buttons($button_type) . $content;
					}
					if ($position == 'after') {
						$content = $content . $this->build_share_buttons($button_type);
					}
				}
			}
		}
		

		return $content;
	} // End add_share_buttons_to_post()

	

	/**
	 * Shortcode for adding the sharing buttons to content or templates
	 * [ess_post], [ess_post share_type="count"], <?php echo do_shortcode('[ess_post share_type="count"]'); ?>
	 *
	 * @access  public
	 * @since   1.0.0
	 * @return  string
	 */
	public function share_post_shortcode( $atts, $content = null ) {
		$options = shortcode_atts( array(
			'share_type' => 'basic'
		), $atts );

		$button_type = $options['share_type'];

		return $this->build_share_buttons($button_type);
	} // End share_post_shortcode()

	/**
	 * Add the social sharing buttons to post media.
	 * @access  public
	 * @since   1.0.0
	 * @return  string
	 */
	public function add_share_buttons_to_media($content) {
		global $post;

		if (get_option('show_media_buttons')) {
	
			$content = do_shortcode($content);

			$wrap_class = 'ess-image-wrap';	
		  
			if ( trim( $content ) != '' ) {

				libxml_use_internal_errors(true);
				$dom = new DOMDocument();
			
				$dom->loadHTML(mb_convert_encoding($content, 'HTML-ENTITIES', 'UTF-8'));
				
				$wrapper_div = $dom->createElement('div');
				$wrapper_div->setAttribute('class', $wrap_class);

				$ImageList = $dom->getElementsByTagName('img');

				foreach ( $ImageList as $key => $Image ) {

					$share_list = $dom->createElement('ul');
					$share_list->setAttribute('class', 'ess-buttons');

					$sites = get_option('ess_media_social_sites');

					$random_id = substr(base64_encode(basename($Image->getAttribute('src'))), 0, 15);

					foreach($sites as $site) {

						$button = $this->get_button_html($site, $post->ID, false, $Image->getAttribute('src'), $random_id);

						$share_item = $dom->createDocumentFragment();
						$share_item->appendXML($button);
						$share_list->appendChild( $share_item );

					}//End social sites foreach

					if ( $Image->parentNode->nodeName == 'a' ) {

						$link_parent = $Image->parentNode;

						$wrap_clone = $wrapper_div->cloneNode();

						$wrap_clone->setAttribute('id', $random_id);

						$link_parent->parentNode->replaceChild($wrap_clone, $link_parent);
						$wrap_clone->appendChild($link_parent);

						$wrap_clone->appendChild( $share_list );
						
					} else {
						
						$wrap_clone = $wrapper_div->cloneNode();

						$wrap_clone->setAttribute('id', $random_id);

						$Image->parentNode->replaceChild($wrap_clone, $Image);
						$wrap_clone->appendChild($Image);

						$wrap_clone->appendChild( $share_list );
						
					}

				}//End Images foreach
			

				//Fixed the issue with additional html tags loading
				$content = preg_replace('/^<!DOCTYPE.+?>/', '', str_replace( array('<html>', '</html>', '<body>', '</body>'), array('', '', '', ''), $dom->saveHTML()));

			}// End if has content

		}//End if media share option on

	   // return the processed content
	   return $content;
	
	} //End add_share_buttons_to_media()

}//End class Easy_Social_Share_Buttons
